@@ -46,22 +46,6 @@ const struct lws_role_ops *available_roles[] = {
 	NULL
 };
 
-const struct lws_event_loop_ops *available_event_libs[] = {
-#if defined(LWS_WITH_POLL)
-	&event_loop_ops_poll,
-#endif
-#if defined(LWS_WITH_LIBUV)
-	&event_loop_ops_uv,
-#endif
-#if defined(LWS_WITH_LIBEVENT)
-	&event_loop_ops_event,
-#endif
-#if defined(LWS_WITH_LIBEV)
-	&event_loop_ops_ev,
-#endif
-	NULL
-};
-
 #if defined(LWS_WITH_ABSTRACT)
 const struct lws_protocols *available_abstract_protocols[] = {
 #if defined(LWS_ROLE_RAW)
@@ -85,6 +69,7 @@ const struct lws_protocols *available_secstream_protocols[] = {
 #if defined(LWS_ROLE_MQTT)
 	&protocol_secstream_mqtt,
 #endif
+	&protocol_secstream_raw,
 	NULL
 };
 #endif
@@ -125,7 +110,9 @@ lws_role_call_alpn_negotiated(struct lws *wsi, const char *alpn)
 	if (!alpn)
 		return 0;
 
+#if !defined(LWS_ESP_PLATFORM)
 	lwsl_info("%s: '%s'\n", __func__, alpn);
+#endif
 
 	LWS_FOR_EVERY_AVAILABLE_ROLE_START(ar)
 		if (ar->alpn && !strcmp(ar->alpn, alpn) && ar->alpn_negotiated)
@@ -144,18 +131,18 @@ lws_role_call_adoption_bind(struct lws *wsi, int type, const char *prot)
 	 * if the vhost is told to bind accepted sockets to a given role,
 	 * then look it up by name and try to bind to the specific role.
 	 */
-	if (lws_check_opt(wsi->vhost->options,
+	if (lws_check_opt(wsi->a.vhost->options,
 			  LWS_SERVER_OPTION_ADOPT_APPLY_LISTEN_ACCEPT_CONFIG) &&
-	    wsi->vhost->listen_accept_role) {
+	    wsi->a.vhost->listen_accept_role) {
 		const struct lws_role_ops *role =
-			lws_role_by_name(wsi->vhost->listen_accept_role);
+			lws_role_by_name(wsi->a.vhost->listen_accept_role);
 
 		if (!prot)
-			prot = wsi->vhost->listen_accept_protocol;
+			prot = wsi->a.vhost->listen_accept_protocol;
 
 		if (!role)
 			lwsl_err("%s: can't find role '%s'\n", __func__,
-				  wsi->vhost->listen_accept_role);
+				  wsi->a.vhost->listen_accept_role);
 
 		if (role && role->adoption_bind) {
 			n = role->adoption_bind(wsi, type, prot);
@@ -174,7 +161,7 @@ lws_role_call_adoption_bind(struct lws *wsi, int type, const char *prot)
 
 		lwsl_warn("%s: adoption bind to role '%s', "
 			  "protocol '%s', type 0x%x, failed\n", __func__,
-			  wsi->vhost->listen_accept_role, prot, type);
+			  wsi->a.vhost->listen_accept_role, prot, type);
 	}
 
 	/*
@@ -310,16 +297,17 @@ int
 lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
 {
 	const struct lws_protocol_vhost_options *pvo, *pvo1;
-	struct lws *wsi = vh->context->pt[0].fake_wsi;
+	lws_fakewsi_def_plwsa(&vh->context->pt[0]);
 	int n;
 
-	wsi->context = vh->context;
-	wsi->vhost = vh;
+	lws_fakewsi_prep_plwsa_ctx(vh->context);
+
+	plwsa->vhost = vh;
 
 	/* initialize supported protocols on this vhost */
 
 	for (n = 0; n < vh->count_protocols; n++) {
-		wsi->protocol = &vh->protocols[n];
+		plwsa->protocol = &vh->protocols[n];
 		if (!vh->protocols[n].name)
 			continue;
 		pvo = lws_vhost_protocol_options(vh, vh->protocols[n].name);
@@ -369,10 +357,10 @@ lws_protocol_init_vhost(struct lws_vhost *vh, int *any)
 		 * inform all the protocols that they are doing their
 		 * one-time initialization if they want to.
 		 *
-		 * NOTE the wsi is all zeros except for the context, vh
-		 * + protocol ptrs so lws_get_context(wsi) etc can work
+		 * NOTE the fakewsi is garbage, except the key pointers that are
+		 * prepared in case the protocol handler wants to touch them
 		 */
-		if (vh->protocols[n].callback(wsi,
+		if (vh->protocols[n].callback((struct lws *)plwsa,
 				LWS_CALLBACK_PROTOCOL_INIT, NULL,
 				(void *)pvo, 0)) {
 			if (vh->protocol_vh_privs[n]) {
@@ -429,8 +417,9 @@ next:
 	context->protocol_init_done = 1;
 
 #if defined(LWS_WITH_SERVER)
-	if (any)
+	if (any) {
 		lws_tls_check_all_cert_lifetimes(context);
+	}
 #endif
 
 	return 0;
@@ -466,8 +455,7 @@ struct lws_vhost *
 lws_create_vhost(struct lws_context *context,
 		 const struct lws_context_creation_info *info)
 {
-	struct lws_vhost *vh = lws_zalloc(sizeof(*vh), "create vhost"),
-			 **vh1 = &context->vhost_list;
+	struct lws_vhost *vh, **vh1 = &context->vhost_list;
 	const struct lws_http_mount *mounts;
 	const struct lws_protocols *pcols = info->protocols;
 #ifdef LWS_WITH_PLUGINS
@@ -485,9 +473,19 @@ lws_create_vhost(struct lws_context *context,
 #endif
 	int n;
 
+
+	vh = lws_zalloc(sizeof(*vh)
+#if defined(LWS_WITH_EVENT_LIBS)
+			+ context->event_loop_ops->evlib_size_vh
+#endif
+			, __func__);
 	if (!vh)
 		return NULL;
 
+#if defined(LWS_WITH_EVENT_LIBS)
+	vh->evlib_vh = (void *)&vh[1];
+#endif
+
 #if LWS_MAX_SMP > 1
 	pthread_mutex_init(&vh->lock, NULL);
 #endif
@@ -511,6 +509,12 @@ lws_create_vhost(struct lws_context *context,
 	vh->iface = info->iface;
 #if !defined(LWS_PLAT_FREERTOS) && !defined(OPTEE_TA) && !defined(WIN32)
 	vh->bind_iface = info->bind_iface;
+#endif
+#if defined(LWS_WITH_CLIENT)
+	if (info->connect_timeout_secs)
+		vh->connect_timeout_secs = info->connect_timeout_secs;
+	else
+		vh->connect_timeout_secs = 20;
 #endif
 	/* apply the context default lws_retry */
 
@@ -674,15 +678,18 @@ lws_create_vhost(struct lws_context *context,
 #ifdef LWS_WITH_PLUGINS
 	if (plugin) {
 		while (plugin) {
-			for (n = 0; n < plugin->caps.count_protocols; n++) {
+			const lws_plugin_protocol_t *plpr =
+				(const lws_plugin_protocol_t *)plugin->hdr;
+
+			for (n = 0; n < plpr->count_protocols; n++) {
 				/*
 				 * for compatibility's sake, no pvo implies
 				 * allow all protocols
 				 */
 				if (f || lws_vhost_protocol_options(vh,
-				    plugin->caps.protocols[n].name)) {
+						plpr->protocols[n].name)) {
 					memcpy(&lwsp[m],
-					       &plugin->caps.protocols[n],
+					       &plpr->protocols[n],
 					       sizeof(struct lws_protocols));
 					m++;
 					vh->count_protocols++;
@@ -821,7 +828,10 @@ lws_create_vhost(struct lws_context *context,
 		goto bail1;
 	}
 #endif
+
+#if defined(LWS_WITH_SYS_ASYNC_DNS)
 	n = !!context->vhost_list;
+#endif
 
 	while (1) {
 		if (!(*vh1)) {
@@ -886,7 +896,7 @@ lws_cancel_service(struct lws_context *context)
 	if (context->being_destroyed1)
 		return;
 
-	lwsl_info("%s\n", __func__);
+	lwsl_debug("%s\n", __func__);
 
 	while (m--) {
 		if (pt->pipe_wsi)
@@ -898,6 +908,8 @@ lws_cancel_service(struct lws_context *context)
 int
 lws_create_event_pipes(struct lws_context *context)
 {
+	struct lws_context_per_thread *pt;
+	size_t s = sizeof(struct lws);
 	struct lws *wsi;
 	int n;
 
@@ -912,24 +924,35 @@ lws_create_event_pipes(struct lws_context *context)
 	n = 0;
 	{
 #endif
-		if (context->pt[n].pipe_wsi)
+		pt = &context->pt[n];
+
+		if (pt->pipe_wsi)
 			return 0;
 
-		wsi = lws_zalloc(sizeof(*wsi), "event pipe wsi");
+#if defined(LWS_WITH_EVENT_LIBS)
+		s += context->event_loop_ops->evlib_size_wsi;
+#endif
+
+		wsi = lws_zalloc(s, "event pipe wsi");
 		if (!wsi) {
 			lwsl_err("%s: Out of mem\n", __func__);
 			return 1;
 		}
-		wsi->context = context;
+#if defined(LWS_WITH_EVENT_LIBS)
+		wsi->evlib_wsi = (uint8_t *)wsi + sizeof(*wsi);
+#endif
+		wsi->a.context = context;
 		lws_role_transition(wsi, 0, LRS_UNCONNECTED, &role_ops_pipe);
-		wsi->protocol = NULL;
+		wsi->a.protocol = NULL;
 		wsi->tsi = n;
-		wsi->vhost = NULL;
+		wsi->a.vhost = NULL;
 		wsi->event_pipe = 1;
 		wsi->desc.sockfd = LWS_SOCK_INVALID;
 		context->pt[n].pipe_wsi = wsi;
 		context->count_wsi_allocated++;
 
+		lws_pt_lock(pt, __func__); /* -------------- pt { */
+
 		if (!lws_plat_pipe_create(wsi)) {
 			/*
 			 * platform code returns 0 if it actually created pipes
@@ -945,14 +968,21 @@ lws_create_event_pipes(struct lws_context *context)
 
 			if (context->event_loop_ops->sock_accept)
 				if (context->event_loop_ops->sock_accept(wsi))
-					return 1;
+					goto bail;
 
 			if (__insert_wsi_socket_into_fds(context, wsi))
-				return 1;
+				goto bail;
 		}
+
+		lws_pt_unlock(pt);
 	}
 
 	return 0;
+
+bail:
+	lws_pt_unlock(pt);
+
+	return 1;
 }
 
 void
@@ -963,17 +993,17 @@ lws_destroy_event_pipe(struct lws *wsi)
 	if (lws_socket_is_valid(wsi->desc.sockfd))
 		__remove_wsi_socket_from_fds(wsi);
 
-	if (!wsi->context->event_loop_ops->destroy_wsi &&
-	    wsi->context->event_loop_ops->wsi_logical_close) {
-		wsi->context->event_loop_ops->wsi_logical_close(wsi);
+	if (!wsi->a.context->event_loop_ops->destroy_wsi &&
+	    wsi->a.context->event_loop_ops->wsi_logical_close) {
+		wsi->a.context->event_loop_ops->wsi_logical_close(wsi);
 		lws_plat_pipe_close(wsi);
 		return;
 	}
 
-	if (wsi->context->event_loop_ops->destroy_wsi)
-		wsi->context->event_loop_ops->destroy_wsi(wsi);
+	if (wsi->a.context->event_loop_ops->destroy_wsi)
+		wsi->a.context->event_loop_ops->destroy_wsi(wsi);
 	lws_plat_pipe_close(wsi);
-	wsi->context->count_wsi_allocated--;
+	wsi->a.context->count_wsi_allocated--;
 	lws_free(wsi);
 }
 
@@ -1091,25 +1121,26 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 	} lws_end_foreach_dll_safe(d, d1);
 #endif
 
+#if defined(LWS_WITH_DEPRECATED_THINGS)
 	/*
 	 * destroy any pending timed events
 	 */
 
 	while (vh->timed_vh_protocol_list)
 		__lws_timed_callback_remove(vh, vh->timed_vh_protocol_list);
-
+#endif
 	/*
 	 * let the protocols destroy the per-vhost protocol objects
 	 */
 
 	memset(&wsi, 0, sizeof(wsi));
-	wsi.context = vh->context;
-	wsi.vhost = vh; /* not a real bound wsi */
+	wsi.a.context = vh->context;
+	wsi.a.vhost = vh; /* not a real bound wsi */
 	protocol = vh->protocols;
 	if (protocol && vh->created_vhost_protocols) {
 		n = 0;
 		while (n < vh->count_protocols) {
-			wsi.protocol = protocol;
+			wsi.a.protocol = protocol;
 
 			if (protocol->callback)
 				protocol->callback(&wsi, LWS_CALLBACK_PROTOCOL_DESTROY,
@@ -1132,8 +1163,10 @@ __lws_vhost_destroy2(struct lws_vhost *vh)
 
 	/* add ourselves to the pending destruction list */
 
-	vh->vhost_next = vh->context->vhost_pending_destruction_list;
-	vh->context->vhost_pending_destruction_list = vh;
+	if (vh->context->vhost_pending_destruction_list != vh) {
+		vh->vhost_next = vh->context->vhost_pending_destruction_list;
+		vh->context->vhost_pending_destruction_list = vh;
+	}
 
 	lwsl_info("%s: %p\n", __func__, vh);
 
@@ -1282,7 +1315,7 @@ lws_check_deferred_free(struct lws_context *context, int tsi, int force)
 				struct lws *wsi = wsi_from_fd(context, pt->fds[n].fd);
 				if (!wsi)
 					continue;
-				if (wsi->vhost != v)
+				if (wsi->a.vhost != v)
 					continue;
 
 				__lws_close_free_wsi(wsi,
@@ -1379,7 +1412,7 @@ lws_context_deprecate(struct lws_context *context, lws_reload_func cb)
 		if (wsi) {
 			wsi->socket_is_permanently_unusable = 1;
 			lws_close_free_wsi(wsi, LWS_CLOSE_STATUS_NOSTATUS, "ctx deprecate");
-			wsi->context->deprecation_pending_listen_close_count++;
+			wsi->a.context->deprecation_pending_listen_close_count++;
 			/*
 			 * other vhosts can share the listen port, they
 			 * point to the same wsi.  So zap those too.
@@ -1423,6 +1456,10 @@ lws_get_vhost_by_name(struct lws_context *context, const char *name)
  *
  * This was originally in the client code but since the list is held on the
  * vhost (to ensure the same client tls ctx is involved) it's cleaner in vhost.c
+ *
+ * ACTIVE_CONNS_QUEUED: We're queued on an active connection, set *nwsi to that
+ * ACTIVE_CONNS_MUXED: We are joining an active mux conn *nwsi as a child
+ * ACTIVE_CONNS_SOLO: There's no existing conn to join either way
  */
 
 int
@@ -1449,10 +1486,10 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 	}
 #endif
 
-	lws_vhost_lock(wsi->vhost); /* ----------------------------------- { */
+	lws_vhost_lock(wsi->a.vhost); /* ----------------------------------- { */
 
 	lws_start_foreach_dll_safe(struct lws_dll2 *, d, d1,
-				   wsi->vhost->dll_cli_active_conns_owner.head) {
+				   wsi->a.vhost->dll_cli_active_conns_owner.head) {
 		struct lws *w = lws_container_of(d, struct lws,
 						 dll_cli_active_conns);
 
@@ -1501,14 +1538,14 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 
 				if (lwsi_state(w) == LRS_IDLING) {
 					// lwsi_set_state(w, LRS_ESTABLISHED);
-					_lws_generic_transaction_completed_active_conn(&w);
+					_lws_generic_transaction_completed_active_conn(&w, 0);
 				}
 
 				//lwsi_set_state(w, LRS_H1C_ISSUE_HANDSHAKE2);
 
 				wsi->client_h2_alpn = 1;
 				lws_wsi_h2_adopt(w, wsi);
-				lws_vhost_unlock(wsi->vhost); /* } ---------- */
+				lws_vhost_unlock(wsi->a.vhost); /* } ---------- */
 
 				*nwsi = w;
 
@@ -1530,7 +1567,7 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 					lws_dll2_remove(&wsi->dll2_cli_txn_queue);
 					wsi->client_mux_substream = 1;
 
-					lws_vhost_unlock(wsi->vhost); /* } ---------- */
+					lws_vhost_unlock(wsi->a.vhost); /* } ---------- */
 
 
 					return ACTIVE_CONNS_MUXED;
@@ -1555,7 +1592,7 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 
 			if (lwsi_state(w) == LRS_IDLING) {
 				// lwsi_set_state(w, LRS_ESTABLISHED);
-				_lws_generic_transaction_completed_active_conn(&w);
+				_lws_generic_transaction_completed_active_conn(&w, 0);
 			}
 
 			/*
@@ -1563,7 +1600,7 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 			 * and wait for our turn at client transaction_complete
 			 * to take over parsing the rx.
 			 */
-			lws_vhost_unlock(wsi->vhost); /* } ---------- */
+			lws_vhost_unlock(wsi->a.vhost); /* } ---------- */
 
 			*nwsi = w;
 
@@ -1573,7 +1610,7 @@ lws_vhost_active_conns(struct lws *wsi, struct lws **nwsi, const char *adsin)
 	} lws_end_foreach_dll_safe(d, d1);
 
 solo:
-	lws_vhost_unlock(wsi->vhost); /* } ---------------------------------- */
+	lws_vhost_unlock(wsi->a.vhost); /* } ---------------------------------- */
 
 	/* there is nobody already connected in the same way */
 
