@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -77,6 +77,7 @@ lejp_construct(struct lejp_ctx *ctx,
 	ctx->st[0].b = 0;
 	ctx->sp = 0;
 	ctx->ipos = 0;
+	ctx->outer_array = 0;
 	ctx->path_match = 0;
 	ctx->path_stride = 0;
 	ctx->path[0] = '\0';
@@ -229,13 +230,15 @@ lejp_get_wildcard(struct lejp_ctx *ctx, int wildcard, char *dest, int len)
  * unused.
  */
 
+static const char esc_char[] = "\"\\/bfnrt";
+static const char esc_tran[] = "\"\\/\b\f\n\r\t";
+static const char tokens[] = "rue alse ull ";
+
 int
 lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 {
-	unsigned char c, n, s, ret = LEJP_REJECT_UNKNOWN;
-	static const char esc_char[] = "\"\\/bfnrt";
-	static const char esc_tran[] = "\"\\/\b\f\n\r\t";
-	static const char tokens[] = "rue alse ull ";
+	unsigned char c, n, s;
+	int ret = LEJP_REJECT_UNKNOWN;
 
 	if (!ctx->sp && !ctx->pst[ctx->pst_sp].ppos)
 		ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_START);
@@ -263,18 +266,37 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 
 		switch (s) {
 		case LEJP_IDLE:
+			if (!ctx->sp && c == '[') {
+				/* push */
+				ctx->outer_array = 1;
+				ctx->st[ctx->sp].s = LEJP_MP_ARRAY_END;
+				c = LEJP_MP_VALUE;
+				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = '[';
+				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = ']';
+				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
+				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_ARRAY_START))
+					goto reject_callback;
+				ctx->i[ctx->ipos++] = 0;
+				if (ctx->ipos > LWS_ARRAY_SIZE(ctx->i)) {
+					ret = LEJP_REJECT_MP_DELIM_ISTACK;
+					goto reject;
+				}
+				goto add_stack_level;
+			}
 			if (c != '{') {
 				ret = LEJP_REJECT_IDLE_NO_BRACE;
 				goto reject;
 			}
-			if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_OBJECT_START)) {
-				ret = LEJP_REJECT_CALLBACK;
-				goto reject;
-			}
+			if (ctx->pst[ctx->pst_sp].callback(ctx,
+							   LEJPCB_OBJECT_START))
+				goto reject_callback;
 			ctx->st[ctx->sp].s = LEJP_MEMBERS;
 			break;
 		case LEJP_MEMBERS:
 			if (c == '}') {
+				if (ctx->sp >= 1)
+					goto pop_level;
+
 				ctx->st[ctx->sp].s = LEJP_IDLE;
 				ret = LEJP_REJECT_MEMBERS_NO_CLOSE;
 				goto reject;
@@ -300,10 +322,8 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				if (ctx->st[ctx->sp - 1].s != LEJP_MP_DELIM) {
 					ctx->buf[ctx->npos] = '\0';
 					if (ctx->pst[ctx->pst_sp].callback(ctx,
-						      LEJPCB_VAL_STR_END) < 0) {
-						ret = LEJP_REJECT_CALLBACK;
-						goto reject;
-					}
+						      LEJPCB_VAL_STR_END) < 0)
+						goto reject_callback;
 				}
 				/* pop */
 				ctx->sp--;
@@ -346,10 +366,10 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->uni |= c - '0';
 			else
 				if (c >= 'a' && c <= 'f')
-					ctx->uni = c - 'a' + 10;
+					ctx->uni |= c - 'a' + 10;
 				else
 					if (c >= 'A' && c <= 'F')
-						ctx->uni = c - 'A' + 10;
+						ctx->uni |= c - 'A' + 10;
 					else {
 						ret = LEJP_REJECT_ILLEGAL_HEX;
 						goto reject;
@@ -409,10 +429,8 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 			ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
 
 			lejp_check_path_match(ctx);
-			if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_PAIR_NAME)) {
-				ret = LEJP_REJECT_CALLBACK;
-				goto reject;
-			}
+			if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_PAIR_NAME))
+				goto reject_callback;
 			break;
 
 		case LEJP_MP_VALUE:
@@ -430,10 +448,9 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				c = LEJP_MP_STRING;
 				ctx->npos = 0;
 				ctx->buf[0] = '\0';
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_STR_START)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+							LEJPCB_VAL_STR_START))
+					goto reject_callback;
 				goto add_stack_level;
 
 			case '{':
@@ -441,10 +458,9 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->st[ctx->sp].s = LEJP_MP_COMMA_OR_END;
 				c = LEJP_MEMBERS;
 				lejp_check_path_match(ctx);
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_OBJECT_START)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+							LEJPCB_OBJECT_START))
+					goto reject_callback;
 				ctx->path_match = 0;
 				goto add_stack_level;
 
@@ -455,10 +471,8 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = '[';
 				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = ']';
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_ARRAY_START)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_ARRAY_START))
+					goto reject_callback;
 				ctx->i[ctx->ipos++] = 0;
 				if (ctx->ipos > LWS_ARRAY_SIZE(ctx->i)) {
 					ret = LEJP_REJECT_MP_DELIM_ISTACK;
@@ -479,7 +493,8 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				}
 				/* drop the path [n] bit */
 				if (ctx->sp) {
-					ctx->pst[ctx->pst_sp].ppos = ctx->st[ctx->sp - 1].p;
+					ctx->pst[ctx->pst_sp].ppos =
+							ctx->st[ctx->sp - 1].p;
 					ctx->ipos = ctx->st[ctx->sp - 1].i;
 				}
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
@@ -490,6 +505,10 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					 * smaller than the matching point
 					 */
 					ctx->path_match = 0;
+				if (ctx->outer_array && !ctx->sp) { /* ended on ] */
+					n = LEJPCB_ARRAY_END;
+					goto completed;
+				}
 				goto array_end;
 
 			case 't': /* true */
@@ -559,15 +578,13 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 
 			ctx->buf[ctx->npos] = '\0';
 			if (ctx->f & LEJP_SEEN_POINT) {
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_NUM_FLOAT)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+							LEJPCB_VAL_NUM_FLOAT))
+					goto reject_callback;
 			} else {
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_NUM_INT)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+							LEJPCB_VAL_NUM_INT))
+					goto reject_callback;
 			}
 
 			/* then this is the post-number character, loop */
@@ -595,25 +612,22 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 			case 3:
 				ctx->buf[0] = '1';
 				ctx->buf[1] = '\0';
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_TRUE)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+								LEJPCB_VAL_TRUE))
+					goto reject_callback;
 				break;
 			case 8:
 				ctx->buf[0] = '0';
 				ctx->buf[1] = '\0';
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_FALSE)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+								LEJPCB_VAL_FALSE))
+					goto reject_callback;
 				break;
 			case 12:
 				ctx->buf[0] = '\0';
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_NULL)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+								LEJPCB_VAL_NULL))
+					goto reject_callback;
 				break;
 			}
 			ctx->st[ctx->sp].s = LEJP_MP_COMMA_OR_END;
@@ -636,7 +650,7 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->pst[ctx->pst_sp].ppos = ctx->st[ctx->sp - 1].p;
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
 				if (ctx->path_match &&
-						ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
+				    ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
 					/*
 					 * we shrank the path to be
 					 * smaller than the matching point
@@ -652,7 +666,7 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				break;
 			}
 			if (c == ']') {
-				if (!ctx->sp) {  /* JSON can't end on ] */
+				if (!ctx->sp) {
 					ret = LEJP_REJECT_MP_C_OR_E_UNDERF;
 					goto reject;
 				}
@@ -662,66 +676,68 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					ret = LEJP_REJECT_MP_C_OR_E_NOTARRAY;
 					goto reject;
 				}
+
 				/* drop the path [n] bit */
 				if (ctx->sp) {
-					ctx->pst[ctx->pst_sp].ppos = ctx->st[ctx->sp - 1].p;
+					ctx->pst[ctx->pst_sp].ppos =
+							ctx->st[ctx->sp - 1].p;
 					ctx->ipos = ctx->st[ctx->sp - 1].i;
 				}
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
 				if (ctx->path_match &&
-						ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
+				    ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
 					/*
 					 * we shrank the path to be
 					 * smaller than the matching point
 					 */
 					ctx->path_match = 0;
 
+				if (ctx->outer_array && !ctx->sp) { /* ended on ] */
+					n = LEJPCB_ARRAY_END;
+					goto completed;
+				}
+
 				/* do LEJP_MP_ARRAY_END processing */
 				goto redo_character;
 			}
-			if (c == '}') {
-				if (!ctx->sp) {
-					lejp_check_path_match(ctx);
-					if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_OBJECT_END)) {
-						ret = LEJP_REJECT_CALLBACK;
-						goto reject;
-					}
-					if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_COMPLETE))
-						goto reject;
-					else
-						/* done, return unused amount */
-						return len;
-				}
-
-				/* pop */
-
-				ctx->sp--;
-				if (ctx->sp) {
-					ctx->pst[ctx->pst_sp].ppos =
-							ctx->st[ctx->sp].p;
-					ctx->ipos = ctx->st[ctx->sp].i;
-				}
-				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
-				if (ctx->path_match &&
-				    ctx->pst[ctx->pst_sp].ppos <=
-						    ctx->path_match_len)
-					/*
-					 * we shrank the path to be
-					 * smaller than the matching point
-					 */
-					ctx->path_match = 0;
-
+			if (c != '}') {
+				ret = LEJP_REJECT_MP_C_OR_E_NEITHER;
+				goto reject;
+			}
+			if (!ctx->sp) {
+				n = LEJPCB_OBJECT_END;
+completed:
 				lejp_check_path_match(ctx);
-				if (ctx->pst[ctx->pst_sp].callback(ctx,
-							LEJPCB_OBJECT_END)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
-				break;
+				if (ctx->pst[ctx->pst_sp].callback(ctx, n) ||
+				    ctx->pst[ctx->pst_sp].callback(ctx,
+							    LEJPCB_COMPLETE))
+					goto reject_callback;
+
+				/* done, return unused amount */
+				return len;
 			}
 
-			ret = LEJP_REJECT_MP_C_OR_E_NEITHER;
-			goto reject;
+			/* pop */
+pop_level:
+			ctx->sp--;
+			if (ctx->sp) {
+				ctx->pst[ctx->pst_sp].ppos = ctx->st[ctx->sp].p;
+				ctx->ipos = ctx->st[ctx->sp].i;
+			}
+			ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
+			if (ctx->path_match &&
+			    ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
+				/*
+				 * we shrank the path to be
+				 * smaller than the matching point
+				 */
+				ctx->path_match = 0;
+
+			lejp_check_path_match(ctx);
+			if (ctx->pst[ctx->pst_sp].callback(ctx,
+							   LEJPCB_OBJECT_END))
+				goto reject_callback;
+			break;
 
 		case LEJP_MP_ARRAY_END:
 array_end:
@@ -732,7 +748,8 @@ array_end:
 					ctx->i[ctx->ipos - 1]++;
 				ctx->st[ctx->sp].s = LEJP_MP_VALUE;
 				if (ctx->sp)
-					ctx->pst[ctx->pst_sp].ppos = ctx->st[ctx->sp - 1].p;
+					ctx->pst[ctx->pst_sp].ppos =
+							ctx->st[ctx->sp - 1].p;
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
 				break;
 			}
@@ -753,10 +770,9 @@ emit_string_char:
 			/* assemble the string value into chunks */
 			ctx->buf[ctx->npos++] = c;
 			if (ctx->npos == sizeof(ctx->buf) - 1) {
-				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_VAL_STR_CHUNK)) {
-					ret = LEJP_REJECT_CALLBACK;
-					goto reject;
-				}
+				if (ctx->pst[ctx->pst_sp].callback(ctx,
+							  LEJPCB_VAL_STR_CHUNK))
+					goto reject_callback;
 				ctx->npos = 0;
 			}
 			continue;
@@ -798,6 +814,10 @@ redo_character:
 
 	return LEJP_CONTINUE;
 
+
+reject_callback:
+	ret = LEJP_REJECT_CALLBACK;
+
 reject:
 	ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_FAILED);
 	return ret;
