@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -60,42 +60,32 @@ struct lws_peer *
 lws_get_or_create_peer(struct lws_vhost *vhost, lws_sockfd_type sockfd)
 {
 	struct lws_context *context = vhost->context;
-	socklen_t rlen = 0;
-	void *q;
-	uint8_t *q8;
 	struct lws_peer *peer;
+	lws_sockaddr46 sa46;
+	socklen_t rlen = 0;
 	uint32_t hash = 0;
-	int n, af = AF_INET;
-	struct sockaddr_storage addr;
+	uint8_t *q8;
+	void *q;
+	int n;
 
 	if (vhost->options & LWS_SERVER_OPTION_UNIX_SOCK)
 		return NULL;
 
-#ifdef LWS_WITH_IPV6
-	if (LWS_IPV6_ENABLED(vhost)) {
-		af = AF_INET6;
-	}
-#endif
-	rlen = sizeof(addr);
-	if (getpeername(sockfd, (struct sockaddr*)&addr, &rlen))
+	rlen = sizeof(sa46);
+	if (getpeername(sockfd, (struct sockaddr*)&sa46, &rlen))
 		/* eg, udp doesn't have to have a peer */
 		return NULL;
 
 #ifdef LWS_WITH_IPV6
-	if (af == AF_INET)
+	if (sa46.sa4.sin_family == AF_INET6) {
+		q = &sa46.sa6.sin6_addr;
+		rlen = sizeof(sa46.sa6.sin6_addr);
+	} else
 #endif
 	{
-		struct sockaddr_in *s = (struct sockaddr_in *)&addr;
-		q = &s->sin_addr;
-		rlen = sizeof(s->sin_addr);
+		q = &sa46.sa4.sin_addr;
+		rlen = sizeof(sa46.sa4.sin_addr);
 	}
-#ifdef LWS_WITH_IPV6
-	else {
-		struct sockaddr_in6 *s = (struct sockaddr_in6 *)&addr;
-		q = &s->sin6_addr;
-		rlen = sizeof(s->sin6_addr);
-	}
-#endif
 
 	q8 = q;
 	for (n = 0; n < (int)rlen; n++)
@@ -107,9 +97,21 @@ lws_get_or_create_peer(struct lws_vhost *vhost, lws_sockfd_type sockfd)
 
 	lws_start_foreach_ll(struct lws_peer *, peerx,
 			     context->pl_hash_table[hash]) {
-		if (peerx->af == af && !memcmp(q, peerx->addr, rlen)) {
-			lws_context_unlock(context); /* === */
-			return peerx;
+		if (peerx->sa46.sa4.sin_family == sa46.sa4.sin_family) {
+#if defined(LWS_WITH_IPV6)
+			if (sa46.sa4.sin_family == AF_INET6 &&
+			    !memcmp(q, &peerx->sa46.sa6.sin6_addr, rlen))
+				goto hit;
+#endif
+			if (sa46.sa4.sin_family == AF_INET &&
+			    !memcmp(q, &peerx->sa46.sa4.sin_addr, rlen)) {
+#if defined(LWS_WITH_IPV6)
+hit:
+#endif
+				lws_context_unlock(context); /* === */
+
+				return peerx;
+			}
 		}
 	} lws_end_foreach_ll(peerx, next);
 
@@ -125,9 +127,8 @@ lws_get_or_create_peer(struct lws_vhost *vhost, lws_sockfd_type sockfd)
 	context->count_peers++;
 	peer->next = context->pl_hash_table[hash];
 	peer->hash = hash;
-	peer->af = af;
+	peer->sa46 = sa46;
 	context->pl_hash_table[hash] = peer;
-	memcpy(peer->addr, q, rlen);
 	time(&peer->time_created);
 	/*
 	 * On creation, the peer has no wsi attached, so is created on the
