@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -26,6 +26,12 @@
 #define _GNU_SOURCE
 #endif
 
+#if defined(WIN32)
+#define HAVE_STRUCT_TIMESPEC
+#if defined(pid_t)
+#undef pid_t
+#endif
+#endif
 #include <pthread.h>
 
 #include "private-lib-core.h"
@@ -36,59 +42,61 @@
 struct lws_threadpool;
 
 struct lws_threadpool_task {
-	struct lws_threadpool_task *task_queue_next;
+	struct lws_threadpool_task	*task_queue_next;
 
-	struct lws_threadpool *tp;
-	char name[32];
+	struct lws_threadpool		*tp;
+	char				name[32];
 	struct lws_threadpool_task_args args;
 
-	lws_usec_t created;
-	lws_usec_t acquired;
-	lws_usec_t done;
-	lws_usec_t entered_state;
+	lws_dll2_t			list;
 
-	lws_usec_t acc_running;
-	lws_usec_t acc_syncing;
+	lws_usec_t			created;
+	lws_usec_t			acquired;
+	lws_usec_t			done;
+	lws_usec_t			entered_state;
 
-	pthread_cond_t wake_idle;
+	lws_usec_t			acc_running;
+	lws_usec_t			acc_syncing;
+
+	pthread_cond_t			wake_idle;
 
 	enum lws_threadpool_task_status status;
 
-	int late_sync_retries;
+	int				late_sync_retries;
 
-	char wanted_writeable_cb;
-	char outlive;
+	char				wanted_writeable_cb;
+	char				outlive;
 };
 
 struct lws_pool {
-	struct lws_threadpool *tp;
-	pthread_t thread;
-	pthread_mutex_t lock; /* part of task wake_idle */
-	struct lws_threadpool_task *task;
-	lws_usec_t acquired;
-	int worker_index;
+	struct lws_threadpool		*tp;
+	pthread_t			thread;
+	pthread_mutex_t			lock; /* part of task wake_idle */
+	struct lws_threadpool_task	*task;
+	lws_usec_t			acquired;
+	int				worker_index;
 };
 
 struct lws_threadpool {
-	pthread_mutex_t lock; /* protects all pool lists */
-	pthread_cond_t wake_idle;
-	struct lws_pool *pool_list;
+	pthread_mutex_t			lock; /* protects all pool lists */
+	pthread_cond_t			wake_idle;
+	struct lws_pool			*pool_list;
 
-	struct lws_context *context;
-	struct lws_threadpool *tp_list; /* context list of threadpools */
+	struct lws_context		*context;
+	struct lws_threadpool		*tp_list; /* context list of threadpools */
 
-	struct lws_threadpool_task *task_queue_head;
-	struct lws_threadpool_task *task_done_head;
+	struct lws_threadpool_task	*task_queue_head;
+	struct lws_threadpool_task	*task_done_head;
 
-	char name[32];
+	char				name[32];
 
-	int threads_in_pool;
-	int queue_depth;
-	int done_queue_depth;
-	int max_queue_depth;
-	int running_tasks;
+	int				threads_in_pool;
+	int				queue_depth;
+	int				done_queue_depth;
+	int				max_queue_depth;
+	int				running_tasks;
 
-	unsigned int destroying:1;
+	unsigned int			destroying:1;
 };
 
 static int
@@ -129,10 +137,10 @@ __lws_threadpool_task_dump(struct lws_threadpool_task *task, char *buf, int len)
 	}
 
 	if (task->acc_running)
-		runms = task->acc_running;
+		runms = (int)task->acc_running;
 
 	if (task->acc_syncing)
-		syncms = task->acc_syncing;
+		syncms = (int)task->acc_syncing;
 
 	if (!task->done) {
 		buf += lws_snprintf(buf, end - buf,
@@ -163,7 +171,8 @@ __lws_threadpool_task_dump(struct lws_threadpool_task *task, char *buf, int len)
 void
 lws_threadpool_dump(struct lws_threadpool *tp)
 {
-#if defined(_DEBUG)
+#if 0
+	//defined(_DEBUG)
 	struct lws_threadpool_task **c;
 	char buf[160];
 	int n, count;
@@ -232,17 +241,26 @@ state_transition(struct lws_threadpool_task *task,
 	task->status = status;
 }
 
+static struct lws *
+task_to_wsi(struct lws_threadpool_task *task)
+{
+#if defined(LWS_WITH_SECURE_STREAMS)
+	if (task->args.ss)
+		return task->args.ss->wsi;
+#endif
+	return task->args.wsi;
+}
+
 static void
 lws_threadpool_task_cleanup_destroy(struct lws_threadpool_task *task)
 {
 	if (task->args.cleanup)
-		task->args.cleanup(task->args.wsi, task->args.user);
+		task->args.cleanup(task_to_wsi(task), task->args.user);
 
-	if (task->args.wsi)
-		task->args.wsi->tp_task = NULL;
+	lws_dll2_remove(&task->list);
 
 	lwsl_thread("%s: tp %p: cleaned finished task for wsi %p\n",
-		    __func__, task->tp, task->args.wsi);
+		    __func__, task->tp, task_to_wsi(task));
 
 	lws_free(task);
 }
@@ -255,25 +273,34 @@ __lws_threadpool_reap(struct lws_threadpool_task *task)
 
 	/* remove the task from the done queue */
 
-	c = &tp->task_done_head;
+	if (tp) {
+		c = &tp->task_done_head;
 
-	while (*c) {
-		if ((*c) == task) {
-			t = *c;
-			*c = t->task_queue_next;
-			t->task_queue_next = NULL;
-			tp->done_queue_depth--;
+		while (*c) {
+			if ((*c) == task) {
+				t = *c;
+				*c = t->task_queue_next;
+				t->task_queue_next = NULL;
+				tp->done_queue_depth--;
 
-			lwsl_thread("%s: tp %s: reaped task wsi %p\n", __func__,
-				   tp->name, task->args.wsi);
+				lwsl_thread("%s: tp %s: reaped task wsi %p\n", __func__,
+					   tp->name, task_to_wsi(task));
 
-			break;
+				break;
+			}
+			c = &(*c)->task_queue_next;
 		}
-		c = &(*c)->task_queue_next;
-	}
 
-	if (!t)
-		lwsl_err("%s: task %p not in done queue\n", __func__, task);
+		if (!t) {
+			lwsl_err("%s: task %p not in done queue\n", __func__, task);
+			/*
+			 * This shouldn't occur, but in this case not really
+			 * safe to assume there's a task to destroy
+			 */
+			return;
+		}
+	} else
+		lwsl_err("%s: task->tp NULL already\n", __func__);
 
 	/* call the task's cleanup and delete the task itself */
 
@@ -308,9 +335,10 @@ lws_threadpool_tsi_context(struct lws_context *context, int tsi)
 			if (!task)
 				continue;
 
-			wsi = task->args.wsi;
+			wsi = task_to_wsi(task);
 			if (!wsi || wsi->tsi != tsi ||
-			    !task->wanted_writeable_cb)
+			    (!task->wanted_writeable_cb &&
+			     task->status != LWS_TP_STATUS_SYNCING))
 				continue;
 
 			task->wanted_writeable_cb = 0;
@@ -331,10 +359,11 @@ lws_threadpool_tsi_context(struct lws_context *context, int tsi)
 
 		while (*c) {
 			task = *c;
-			wsi = task->args.wsi;
+			wsi = task_to_wsi(task);
 
 			if (wsi && wsi->tsi == tsi &&
-			    task->wanted_writeable_cb) {
+			    (task->wanted_writeable_cb ||
+			     task->status == LWS_TP_STATUS_SYNCING)) {
 
 				task->wanted_writeable_cb = 0;
 				lws_memory_barrier();
@@ -373,12 +402,12 @@ lws_threadpool_worker_sync(struct lws_pool *pool,
 	pthread_mutex_lock(&pool->lock); /* ======================= pool lock */
 
 	lwsl_info("%s: %s: task %p (%s): syncing with wsi %p\n", __func__,
-		    pool->tp->name, task, task->name, task->args.wsi);
+		    pool->tp->name, task, task->name, task_to_wsi(task));
 
 	temp = task->status;
 	state_transition(task, LWS_TP_STATUS_SYNCING);
 	while (tries--) {
-		wsi = task->args.wsi;
+		wsi = task_to_wsi(task);
 
 		/*
 		 * if the wsi is no longer attached to this task, there is
@@ -431,7 +460,7 @@ lws_threadpool_worker_sync(struct lws_pool *pool,
 				lwsl_err("%s: %s: task %p (%s): SYNC timed out "
 					 "(associated wsi %p)\n",
 					 __func__, pool->tp->name, task,
-					 task->name, task->args.wsi);
+					 task->name, task_to_wsi(task));
 
 				state_transition(task, LWS_TP_STATUS_STOPPING);
 				goto done;
@@ -453,6 +482,10 @@ done:
 	return 0;
 }
 
+#if !defined(WIN32)
+static int dummy;
+#endif
+
 static void *
 lws_threadpool_worker(void *d)
 {
@@ -475,8 +508,8 @@ lws_threadpool_worker(void *d)
 			pthread_cond_wait(&tp->wake_idle, &tp->lock);
 
 		if (tp->destroying) {
-			pthread_mutex_unlock(&tp->lock);  /* ------ tp unlock */
-			continue;
+			lwsl_notice("%s: bailing\n", __func__);
+			goto doneski;
 		}
 
 		c = &tp->task_queue_head;
@@ -547,7 +580,7 @@ lws_threadpool_worker(void *d)
 			lws_usec_t then;
 			int n;
 
-			if (tp->destroying || !task->args.wsi) {
+			if (tp->destroying || !task_to_wsi(task)) {
 				lwsl_info("%s: stopping on wsi gone\n", __func__);
 				state_transition(task, LWS_TP_STATUS_STOPPING);
 			}
@@ -563,7 +596,7 @@ lws_threadpool_worker(void *d)
 				/* if not destroying the tp, continue */
 				break;
 			case LWS_TP_RETURN_SYNC:
-				if (!task->args.wsi) {
+				if (!task_to_wsi(task)) {
 					lwsl_debug("%s: task that wants to "
 						    "outlive lost wsi asked "
 						    "to sync: bypassed\n",
@@ -623,21 +656,25 @@ lws_threadpool_worker(void *d)
 			/* signal the associated wsi to take a fresh look at
 			 * task status */
 
-			if (pool->task->args.wsi) {
+			if (task_to_wsi(pool->task)) {
 				task->wanted_writeable_cb = 1;
 
 				lws_cancel_service(
-					lws_get_context(pool->task->args.wsi));
+					lws_get_context(task_to_wsi(pool->task)));
 			}
 		}
 
+doneski:
 		pool->task = NULL;
 		pthread_mutex_unlock(&tp->lock); /* --------------- tp unlock */
 	}
 
-	/* threadpool is being destroyed */
+	lwsl_notice("%s: Exiting\n", __func__);
 
-	pthread_exit(NULL);
+	/* threadpool is being destroyed */
+#if !defined(WIN32)
+	pthread_exit(&dummy);
+#endif
 
 	return NULL;
 }
@@ -724,9 +761,8 @@ lws_threadpool_finish(struct lws_threadpool *tp)
 		c = &task->task_queue_next;
 	}
 
-	pthread_mutex_unlock(&tp->lock); /* -------------------- tpool unlock */
-
 	pthread_cond_broadcast(&tp->wake_idle);
+	pthread_mutex_unlock(&tp->lock); /* -------------------- tpool unlock */
 }
 
 void
@@ -740,8 +776,8 @@ lws_threadpool_destroy(struct lws_threadpool *tp)
 	/* remove us from the context list of threadpools */
 
 	lws_context_lock(tp->context, __func__);
-
 	ptp = &tp->context->tp_list_head;
+
 	while (*ptp) {
 		if (*ptp == tp) {
 			*ptp = tp->tp_list;
@@ -752,27 +788,32 @@ lws_threadpool_destroy(struct lws_threadpool *tp)
 
 	lws_context_unlock(tp->context);
 
+	/*
+	 * Wake up the threadpool guys and tell them to exit
+	 */
 
 	pthread_mutex_lock(&tp->lock); /* ======================== tpool lock */
-
 	tp->destroying = 1;
 	pthread_cond_broadcast(&tp->wake_idle);
 	pthread_mutex_unlock(&tp->lock); /* -------------------- tpool unlock */
 
 	lws_threadpool_dump(tp);
 
+	lwsl_info("%s: waiting for threads to rejoin\n", __func__);
+#if defined(WIN32)
+	Sleep(1000);
+#endif
+
 	for (n = 0; n < tp->threads_in_pool; n++) {
 		task = tp->pool_list[n].task;
 
-		/* he could be sitting waiting for SYNC */
-
-		if (task != NULL)
-			pthread_cond_broadcast(&task->wake_idle);
-
 		pthread_join(tp->pool_list[n].thread, &retval);
 		pthread_mutex_destroy(&tp->pool_list[n].lock);
 	}
 	lwsl_info("%s: all threadpools exited\n", __func__);
+#if defined(WIN32)
+	Sleep(1000);
+#endif
 
 	task = tp->task_done_head;
 	while (task) {
@@ -784,25 +825,21 @@ lws_threadpool_destroy(struct lws_threadpool *tp)
 
 	pthread_mutex_destroy(&tp->lock);
 
+	memset(tp, 0xdd, sizeof(*tp));
 	lws_free(tp);
 }
 
 /*
- * we want to stop and destroy the task and related priv.  The wsi may no
- * longer exist.
+ * We want to stop and destroy the tasks and related priv.
  */
 
 int
-lws_threadpool_dequeue(struct lws *wsi)
+lws_threadpool_dequeue_task(struct lws_threadpool_task *task)
 {
 	struct lws_threadpool *tp;
-	struct lws_threadpool_task **c, *task;
+	struct lws_threadpool_task **c;
 	int n;
 
-	task = wsi->tp_task;
-	if (!task)
-		return 0;
-
 	tp = task->tp;
 	pthread_mutex_lock(&tp->lock); /* ======================== tpool lock */
 
@@ -810,8 +847,11 @@ lws_threadpool_dequeue(struct lws *wsi)
 
 		/* disconnect from wsi, and wsi from task */
 
-		wsi->tp_task = NULL;
+		lws_dll2_remove(&task->list);
 		task->args.wsi = NULL;
+#if defined(LWS_WITH_SECURE_STREAMS)
+		task->args.ss = NULL;
+#endif
 
 		goto bail;
 	}
@@ -833,7 +873,7 @@ lws_threadpool_dequeue(struct lws *wsi)
 			task->done = lws_now_usecs();
 
 			lwsl_debug("%s: tp %p: removed queued task wsi %p\n",
-				    __func__, tp, task->args.wsi);
+				    __func__, tp, task_to_wsi(task));
 
 			break;
 		}
@@ -875,13 +915,16 @@ lws_threadpool_dequeue(struct lws *wsi)
 
 		/* disconnect from wsi, and wsi from task */
 
-		task->args.wsi->tp_task = NULL;
+		lws_dll2_remove(&task->list);
 		task->args.wsi = NULL;
+#if defined(LWS_WITH_SECURE_STREAMS)
+		task->args.ss = NULL;
+#endif
 
 		pthread_mutex_unlock(&tp->pool_list[n].lock);
 
 		lwsl_debug("%s: tp %p: request stop running task "
-			    "for wsi %p\n", __func__, tp, task->args.wsi);
+			    "for wsi %p\n", __func__, tp, task_to_wsi(task));
 
 		break;
 	}
@@ -889,9 +932,12 @@ lws_threadpool_dequeue(struct lws *wsi)
 	if (n == tp->threads_in_pool) {
 		/* can't find it */
 		lwsl_notice("%s: tp %p: no task for wsi %p, decoupling\n",
-			    __func__, tp, task->args.wsi);
-		task->args.wsi->tp_task = NULL;
+			    __func__, tp, task_to_wsi(task));
+		lws_dll2_remove(&task->list);
 		task->args.wsi = NULL;
+#if defined(LWS_WITH_SECURE_STREAMS)
+		task->args.ss = NULL;
+#endif
 	}
 
 bail:
@@ -900,6 +946,21 @@ bail:
 	return 0;
 }
 
+int
+lws_threadpool_dequeue(struct lws *wsi) /* deprecated */
+{
+	struct lws_threadpool_task *task;
+
+	if (!wsi->tp_task_owner.count)
+		return 0;
+	assert(wsi->tp_task_owner.count != 1);
+
+	task = lws_container_of(wsi->tp_task_owner.head,
+				struct lws_threadpool_task, list);
+
+	return lws_threadpool_dequeue_task(task);
+}
+
 struct lws_threadpool_task *
 lws_threadpool_enqueue(struct lws_threadpool *tp,
 		       const struct lws_threadpool_task_args *args,
@@ -911,6 +972,10 @@ lws_threadpool_enqueue(struct lws_threadpool *tp,
 	if (tp->destroying)
 		return NULL;
 
+#if defined(LWS_WITH_SECURE_STREAMS)
+	assert(args->ss || args->wsi);
+#endif
+
 	pthread_mutex_lock(&tp->lock); /* ======================== tpool lock */
 
 	/*
@@ -957,10 +1022,15 @@ lws_threadpool_enqueue(struct lws_threadpool *tp,
 	 * whatever reason can clean up)
 	 */
 
-	args->wsi->tp_task = task;
+#if defined(LWS_WITH_SECURE_STREAMS)
+	if (args->ss)
+		lws_dll2_add_tail(&task->list, &args->ss->wsi->tp_task_owner);
+	else
+#endif
+		lws_dll2_add_tail(&task->list, &args->wsi->tp_task_owner);
 
 	lwsl_thread("%s: tp %s: enqueued task %p (%s) for wsi %p, depth %d\n",
-		    __func__, tp->name, task, task->name, args->wsi,
+		    __func__, tp->name, task, task->name, task_to_wsi(task),
 		    tp->queue_depth);
 
 	/* alert any idle thread there's something new on the task list */
@@ -977,29 +1047,26 @@ bail:
 /* this should be called from the service thread */
 
 enum lws_threadpool_task_status
-lws_threadpool_task_status_wsi(struct lws *wsi,
-			       struct lws_threadpool_task **task, void **user)
+lws_threadpool_task_status(struct lws_threadpool_task *task, void **user)
 {
 	enum lws_threadpool_task_status status;
-	struct lws_threadpool *tp;
+	struct lws_threadpool *tp = task->tp;
 
-	*task = wsi->tp_task;
-	if (!*task)
-		return -1;
+	if (!tp)
+		return LWS_TP_STATUS_FINISHED;
 
-	tp = (*task)->tp;
-	*user = (*task)->args.user;
-	status = (*task)->status;
+	*user = task->args.user;
+	status = task->status;
 
 	if (status == LWS_TP_STATUS_FINISHED ||
 	    status == LWS_TP_STATUS_STOPPED) {
 		char buf[160];
 
 		pthread_mutex_lock(&tp->lock); /* ================ tpool lock */
-		__lws_threadpool_task_dump(*task, buf, sizeof(buf));
+		__lws_threadpool_task_dump(task, buf, sizeof(buf));
 		lwsl_thread("%s: %s: service thread REAPING: %s\n",
 			    __func__, tp->name, buf);
-		__lws_threadpool_reap(*task);
+		__lws_threadpool_reap(task);
 		lws_memory_barrier();
 		pthread_mutex_unlock(&tp->lock); /* ------------ tpool unlock */
 	}
@@ -1007,13 +1074,107 @@ lws_threadpool_task_status_wsi(struct lws *wsi,
 	return status;
 }
 
+enum lws_threadpool_task_status
+lws_threadpool_task_status_noreap(struct lws_threadpool_task *task)
+{
+	return task->status;
+}
+
+enum lws_threadpool_task_status
+lws_threadpool_task_status_wsi(struct lws *wsi,
+			       struct lws_threadpool_task **_task, void **user)
+{
+	struct lws_threadpool_task *task;
+
+	if (!wsi->tp_task_owner.count) {
+		lwsl_notice("%s: wsi has no task, ~=FINISHED\n", __func__);
+		return LWS_TP_STATUS_FINISHED;
+	}
+
+	assert(wsi->tp_task_owner.count == 1); /* see deprecation docs in hdr */
+
+	task = lws_container_of(wsi->tp_task_owner.head,
+				struct lws_threadpool_task, list);
+
+	*_task = task;
+
+	return lws_threadpool_task_status(task, user);
+}
+
 void
 lws_threadpool_task_sync(struct lws_threadpool_task *task, int stop)
 {
 	lwsl_debug("%s\n", __func__);
+	if (!task)
+		return;
 
 	if (stop)
 		state_transition(task, LWS_TP_STATUS_STOPPING);
 
+	pthread_mutex_lock(&task->tp->lock);
 	pthread_cond_signal(&task->wake_idle);
+	pthread_mutex_unlock(&task->tp->lock);
 }
+
+int
+lws_threadpool_foreach_task_wsi(struct lws *wsi, void *user,
+				int (*cb)(struct lws_threadpool_task *task,
+					  void *user))
+{
+	struct lws_threadpool_task *task1;
+
+	if (wsi->tp_task_owner.head == NULL)
+		return 0;
+
+	task1 = lws_container_of(wsi->tp_task_owner.head,
+				 struct lws_threadpool_task, list);
+
+	pthread_mutex_lock(&task1->tp->lock); /* ================ tpool lock */
+
+	lws_start_foreach_dll_safe(struct lws_dll2 *, d, d1,
+				   wsi->tp_task_owner.head) {
+		struct lws_threadpool_task *task = lws_container_of(d,
+					struct lws_threadpool_task, list);
+
+		if (cb(task, user)) {
+			pthread_mutex_unlock(&task1->tp->lock); /* ------------ tpool unlock */
+			return 1;
+		}
+
+	} lws_end_foreach_dll_safe(d, d1);
+
+	pthread_mutex_unlock(&task1->tp->lock); /* ------------ tpool unlock */
+
+	return 0;
+}
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+int
+lws_threadpool_foreach_task_ss(struct lws_ss_handle *ss, void *user,
+			       int (*cb)(struct lws_threadpool_task *task,
+					 void *user))
+{
+	if (!ss->wsi)
+		return 0;
+
+	return lws_threadpool_foreach_task_wsi(ss->wsi, user, cb);
+}
+#endif
+
+struct lws_threadpool_task *
+lws_threadpool_get_task_wsi(struct lws *wsi)
+{
+	if (wsi->tp_task_owner.head == NULL)
+		return NULL;
+
+	return lws_container_of(wsi->tp_task_owner.head,
+				 struct lws_threadpool_task, list);
+}
+
+#if defined(LWS_WITH_SECURE_STREAMS)
+struct lws_threadpool_task *
+lws_threadpool_get_task_ss(struct lws_ss_handle *ss)
+{
+	return lws_threadpool_get_task_wsi(ss->wsi);
+}
+#endif
