@@ -27,6 +27,18 @@
 #endif
 #include "private-lib-core.h"
 
+/*
+ * Normally you don't want this, use lws_sul instead inside the event loop.
+ * But sometimes for drivers it makes sense, so there's an internal-only
+ * crossplatform api for it.
+ */
+
+void
+lws_msleep(unsigned int ms)
+{
+        usleep(ms * LWS_US_PER_MS);
+}
+
 lws_usec_t
 lws_now_usecs(void)
 {
@@ -89,7 +101,8 @@ lws_plat_write_cert(struct lws_vhost *vhost, int is_key, int fd, void *buf,
 
 	n = write(fd, buf, len);
 
-	fsync(fd);
+	if (fsync(fd))
+		return 1;
 	if (lseek(fd, 0, SEEK_SET) < 0)
 		return 1;
 
