@@ -87,8 +87,10 @@ lws_plat_init(struct lws_context *context,
 	}
 
 	while (n--) {
+               int m;
 		pt->fds_count = 0;
-		pt->events = WSACreateEvent(); /* the cancel event */
+               for (m = 0; m < WSA_MAXIMUM_WAIT_EVENTS; m++)
+                       pt->events[m] = WSACreateEvent();
 		InitializeCriticalSection(&pt->interrupt_lock);
 
 		pt++;
@@ -113,7 +115,9 @@ lws_plat_context_early_destroy(struct lws_context *context)
 	int n = context->count_threads;
 
 	while (n--) {
-		WSACloseEvent(pt->events);
+		int m;
+		for (m = 0; m < WSA_MAXIMUM_WAIT_EVENTS; m++)
+			WSACloseEvent(pt->events[m]);
 		DeleteCriticalSection(&pt->interrupt_lock);
 		pt++;
 	}
