@@ -38,10 +38,15 @@ lws_plat_pipe_signal(struct lws *wsi)
 {
 	struct lws_context_per_thread *pt = &wsi->a.context->pt[(int)wsi->tsi];
 
+	/*
+	 * We need the critical section so that we are either setting it or
+	 * clearing it, no matter how many threads competing there is a clear
+	 * atomic state for the event
+	 */
+
 	EnterCriticalSection(&pt->interrupt_lock);
-	pt->interrupt_requested = 1;
+	WSASetEvent(pt->events[0]); /* trigger the cancel event */
 	LeaveCriticalSection(&pt->interrupt_lock);
-	WSASetEvent(pt->events); /* trigger the cancel event */
 
 	return 0;
 }
