@@ -203,7 +203,7 @@ read:
 		ebuf.token = pt->serv_buf;
 		ebuf.len = lws_ssl_capable_read(wsi,
 					ebuf.token,
-					wsi->context->pt_serv_buf_size);
+					wsi->a.context->pt_serv_buf_size);
 		switch (ebuf.len) {
 		case 0:
 			lwsl_info("%s: zero length read\n", __func__);
@@ -248,7 +248,7 @@ drain:
 		 * callback and drain / re-enable it there
 		 */
 		if (user_callback_handle_rxflow(
-				wsi->protocol->callback,
+				wsi->a.protocol->callback,
 				wsi, LWS_CALLBACK_RECEIVE_CLIENT_HTTP,
 				wsi->user_space, NULL, 0)) {
 			lwsl_info("RECEIVE_CLIENT_HTTP closed it\n");
@@ -403,8 +403,8 @@ rops_write_role_protocol_h2(struct lws *wsi, unsigned char *buf, size_t len,
 	)) {
 		//assert(0);
 		lwsl_notice("%s: binning wsistate 0x%x %d: %s\n", __func__,
-				(unsigned int)wsi->wsistate, *wp, wsi->protocol ?
-					wsi->protocol->name : "no protocol");
+				(unsigned int)wsi->wsistate, *wp, wsi->a.protocol ?
+					wsi->a.protocol->name : "no protocol");
 
 		return 0;
 	}
@@ -488,6 +488,7 @@ rops_write_role_protocol_h2(struct lws *wsi, unsigned char *buf, size_t len,
 	return (int)olen;
 }
 
+#if defined(LWS_WITH_SERVER)
 static int
 rops_check_upgrades_h2(struct lws *wsi)
 {
@@ -501,7 +502,7 @@ rops_check_upgrades_h2(struct lws *wsi)
 	 * SETTINGS saying that we support it though.
 	 */
 	p = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_METHOD);
-	if (!wsi->vhost->h2.set.s[H2SET_ENABLE_CONNECT_PROTOCOL] ||
+	if (!wsi->a.vhost->h2.set.s[H2SET_ENABLE_CONNECT_PROTOCOL] ||
 	    !wsi->mux_substream || !p || strcmp(p, "CONNECT"))
 		return LWS_UPG_RET_CONTINUE;
 
@@ -510,7 +511,7 @@ rops_check_upgrades_h2(struct lws *wsi)
 		return LWS_UPG_RET_CONTINUE;
 
 #if defined(LWS_WITH_SERVER_STATUS)
-	wsi->vhost->conn_stats.ws_upg++;
+	wsi->a.vhost->conn_stats.ws_upg++;
 #endif
 	lwsl_info("Upgrade h2 to ws\n");
 	lws_mux_mark_immortal(wsi);
@@ -526,6 +527,7 @@ rops_check_upgrades_h2(struct lws *wsi)
 	return LWS_UPG_RET_CONTINUE;
 #endif
 }
+#endif
 
 static int
 rops_init_vhost_h2(struct lws_vhost *vh,
@@ -558,8 +560,8 @@ rops_pt_init_destroy_h2(struct lws_context *context,
 
 		pt->sul_ah_lifecheck.cb = lws_sul_http_ah_lifecheck;
 
-		__lws_sul_insert(&pt->pt_sul_owner, &pt->sul_ah_lifecheck,
-				 30 * LWS_US_PER_SEC);
+		__lws_sul_insert_us(&pt->pt_sul_owner[LWSSULLI_MISS_IF_SUSPENDED],
+				 &pt->sul_ah_lifecheck, 30 * LWS_US_PER_SEC);
 	} else
 		lws_dll2_remove(&pt->sul_ah_lifecheck.list);
 #endif
@@ -607,7 +609,7 @@ rops_tx_credit_h2(struct lws *wsi, char peer_to_us, int add)
 static int
 rops_destroy_role_h2(struct lws *wsi)
 {
-	struct lws_context_per_thread *pt = &wsi->context->pt[(int)wsi->tsi];
+	struct lws_context_per_thread *pt = &wsi->a.context->pt[(int)wsi->tsi];
 	struct allocated_headers *ah;
 
 	/* we may not have an ah, but may be on the waiting list... */
@@ -650,7 +652,7 @@ rops_close_kill_connection_h2(struct lws *wsi, enum lws_close_status reason)
 
 		wsi->http.proxy_clientside = 0;
 
-		if (user_callback_handle_rxflow(wsi->protocol->callback,
+		if (user_callback_handle_rxflow(wsi->a.protocol->callback,
 						wsi,
 					    LWS_CALLBACK_COMPLETED_CLIENT_HTTP,
 						wsi->user_space, NULL, 0))
@@ -766,12 +768,22 @@ lws_h2_bind_for_post_before_action(struct lws *wsi)
 
 	p = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_METHOD);
 	if (p && !strcmp(p, "POST")) {
-		const struct lws_http_mount *hit =
-				lws_find_mount(wsi,
-					lws_hdr_simple_ptr(wsi,
-					    WSI_TOKEN_HTTP_COLON_PATH),
-					lws_hdr_total_length(wsi,
-					    WSI_TOKEN_HTTP_COLON_PATH));
+		const struct lws_http_mount *hit;
+
+		if (!lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_PATH) ||
+		    !lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH))
+			/*
+			 * There must be a path.  Actually this is checked at
+			 * http2.c along with the other required header
+			 * presence before we can get here.
+			 *
+			 * But Coverity insists to see us check it.
+			 */
+			return 1;
+
+		hit = lws_find_mount(wsi,
+			  lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH),
+			  lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_PATH));
 
 		lwsl_debug("%s: %s: hit %p: %s\n", __func__,
 			    lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH),
@@ -783,7 +795,7 @@ lws_h2_bind_for_post_before_action(struct lws *wsi)
 			if (hit->protocol)
 				name = hit->protocol;
 
-			pp = lws_vhost_name_to_protocol(wsi->vhost, name);
+			pp = lws_vhost_name_to_protocol(wsi->a.vhost, name);
 			if (!pp) {
 				lwsl_info("Unable to find protocol '%s'\n", name);
 				return 1;
@@ -794,7 +806,7 @@ lws_h2_bind_for_post_before_action(struct lws *wsi)
 		}
 
 		lwsl_info("%s: setting LRS_BODY from 0x%x (%s)\n", __func__,
-			    (int)wsi->wsistate, wsi->protocol->name);
+			    (int)wsi->wsistate, wsi->a.protocol->name);
 		lwsi_set_state(wsi, LRS_BODY);
 	}
 
@@ -1153,7 +1165,7 @@ rops_alpn_negotiated_h2(struct lws *wsi, const char *alpn)
 
 	wsi->upgraded_to_http2 = 1;
 #if defined(LWS_WITH_SERVER_STATUS)
-	wsi->vhost->conn_stats.h2_alpn++;
+	wsi->a.vhost->conn_stats.h2_alpn++;
 #endif
 
 	/* adopt the header info */
@@ -1225,7 +1237,11 @@ rops_issue_keepalive_h2(struct lws *wsi, int isvalid)
 const struct lws_role_ops role_ops_h2 = {
 	/* role name */			"h2",
 	/* alpn id */			"h2",
+#if defined(LWS_WITH_SERVER)
 	/* check_upgrades */		rops_check_upgrades_h2,
+#else
+					NULL,
+#endif
 	/* pt_init_destroy */		rops_pt_init_destroy_h2,
 	/* init_vhost */		rops_init_vhost_h2,
 	/* destroy_vhost */		NULL,
