@@ -30,8 +30,14 @@ rops_handle_POLLIN_pipe(struct lws_context_per_thread *pt, struct lws *wsi,
 {
 #if defined(LWS_HAVE_EVENTFD)
 	eventfd_t value;
-	if (eventfd_read(wsi->desc.sockfd, &value) < 0)
+	int n;
+
+	n = eventfd_read(wsi->desc.sockfd, &value);
+	if (n < 0) {
+		lwsl_notice("%s: eventfd read %d bailed errno %d\n", __func__,
+				wsi->desc.sockfd, LWS_ERRNO);
 		return LWS_HPI_RET_PLEASE_CLOSE_ME;
+	}
 #elif !defined(WIN32) && !defined(_WIN32)
 	char s[100];
 	int n;
