@@ -181,6 +181,7 @@ callback_sspc_client(struct lws *wsi, enum lws_callback_reasons reason,
 		case LWSSSSRET_DISCONNECT_ME:
 			return -1;
 		case LWSSSSRET_DESTROY_ME:
+			lws_set_opaque_user_data(wsi, NULL);
 			lws_sspc_destroy(&h);
 			return -1;
 		}
@@ -511,12 +512,8 @@ lws_sspc_destroy(lws_sspc_handle_t **ph)
 
 	if (h->dsh)
 		lws_dsh_destroy(&h->dsh);
-	if (h->cwsi) {
-		struct lws *wsi = h->cwsi;
+	if (h->cwsi)
 		h->cwsi = NULL;
-		if (wsi)
-			lws_set_timeout(wsi, 1, LWS_TO_KILL_SYNC);
-	}
 
 	/* clean out any pending metadata changes that didn't make it */
 
