@@ -1,36 +1,35 @@
 /*
- * libwebsockets - client-related ssl code independent of backend
+ * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010-2018 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2019 Andy Green <andy@warmcat.com>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation:
- *  version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA  02110-1301  USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
  */
 
-#include "core/private.h"
+#include "private-lib-core.h"
 
 int
-lws_ssl_client_connect1(struct lws *wsi)
+lws_ssl_client_connect1(struct lws *wsi, char *errbuf, int len)
 {
-	struct lws_context *context = wsi->context;
-	int n = 0;
-
-	lws_latency_pre(context, wsi);
-	n = lws_tls_client_connect(wsi);
-	lws_latency(context, wsi, "SSL_connect hs", n, n > 0);
+	int n;
 
+	n = lws_tls_client_connect(wsi, errbuf, len);
 	switch (n) {
 	case LWS_SSL_CAPABLE_ERROR:
 		return -1;
@@ -51,19 +50,15 @@ lws_ssl_client_connect1(struct lws *wsi)
 int
 lws_ssl_client_connect2(struct lws *wsi, char *errbuf, int len)
 {
-	int n = 0;
+	int n;
 
 	if (lwsi_state(wsi) == LRS_WAITING_SSL) {
-		lws_latency_pre(wsi->context, wsi);
-
-		n = lws_tls_client_connect(wsi);
+		n = lws_tls_client_connect(wsi, errbuf, len);
 		lwsl_debug("%s: SSL_connect says %d\n", __func__, n);
-		lws_latency(wsi->context, wsi,
-			    "SSL_connect LRS_WAITING_SSL", n, n > 0);
 
 		switch (n) {
 		case LWS_SSL_CAPABLE_ERROR:
-			lws_snprintf(errbuf, len, "client connect failed");
+			// lws_snprintf(errbuf, len, "client connect failed");
 			return -1;
 		case LWS_SSL_CAPABLE_DONE:
 			break; /* connected */
@@ -92,7 +87,9 @@ int lws_context_init_client_ssl(const struct lws_context_creation_info *info,
 	const char *cert_filepath = info->ssl_cert_filepath;
 	const char *ca_filepath = info->ssl_ca_filepath;
 	const char *cipher_list = info->ssl_cipher_list;
-	struct lws wsi;
+	lws_fakewsi_def_plwsa(&vhost->context->pt[0]);
+
+	lws_fakewsi_prep_plwsa_ctx(vhost->context);
 
 	if (vhost->options & LWS_SERVER_OPTION_ADOPT_APPLY_LISTEN_ACCEPT_CONFIG)
 		return 0;
@@ -123,6 +120,7 @@ int lws_context_init_client_ssl(const struct lws_context_creation_info *info,
 	if (vhost->tls.ssl_client_ctx)
 		return 0;
 
+#if !defined(LWS_WITH_MBEDTLS)
 	if (info->provided_client_ssl_ctx) {
 		/* use the provided OpenSSL context if given one */
 		vhost->tls.ssl_client_ctx = info->provided_client_ssl_ctx;
@@ -131,6 +129,7 @@ int lws_context_init_client_ssl(const struct lws_context_creation_info *info,
 
 		return 0;
 	}
+#endif
 
 	if (lws_tls_client_create_vhost_context(vhost, info, cipher_list,
 						ca_filepath,
@@ -139,7 +138,10 @@ int lws_context_init_client_ssl(const struct lws_context_creation_info *info,
 						cert_filepath,
 						info->client_ssl_cert_mem,
 						info->client_ssl_cert_mem_len,
-						private_key_filepath))
+						private_key_filepath,
+						info->client_ssl_key_mem,
+						info->client_ssl_key_mem_len
+						))
 		return 1;
 
 	lwsl_info("created client ssl context for %s\n", vhost->name);
@@ -148,13 +150,13 @@ int lws_context_init_client_ssl(const struct lws_context_creation_info *info,
 	 * give him a fake wsi with context set, so he can use
 	 * lws_get_context() in the callback
 	 */
-	memset(&wsi, 0, sizeof(wsi));
-	wsi.vhost = vhost; /* not a real bound wsi */
-	wsi.context = vhost->context;
 
-	vhost->protocols[0].callback(&wsi,
+	plwsa->vhost = vhost; /* not a real bound wsi */
+
+	vhost->protocols[0].callback((struct lws *)plwsa,
 			LWS_CALLBACK_OPENSSL_LOAD_EXTRA_CLIENT_VERIFY_CERTS,
 				     vhost->tls.ssl_client_ctx, NULL, 0);
 
 	return 0;
 }
+
