@@ -1,7 +1,7 @@
 /*
  * lws-api-test-lws_struct-json
  *
- * Written in 2010-2019 by Andy Green <andy@warmcat.com>
+ * Written in 2010-2020 by Andy Green <andy@warmcat.com>
  *
  * This file is made available under the Creative Commons CC0 1.0
  * Universal Public Domain Dedication.
@@ -16,6 +16,147 @@
 
 #include <libwebsockets.h>
 
+typedef struct {
+	lws_dll2_t		list;
+
+	struct gpiod_line	*line;
+
+	const char		*name;
+	const char		*wire;
+
+	int			chip_idx;
+	int			offset;
+	int			safe;
+} sai_jig_gpio_t;
+
+typedef struct {
+	lws_dll2_t		list;
+	sai_jig_gpio_t		*gpio; /* null = wait ms */
+	const char		*gpio_name;
+	int			value;
+} sai_jig_seq_item_t;
+
+typedef struct {
+	lws_dll2_t		list;
+	lws_dll2_owner_t	seq_owner;
+	const char		*name;
+} sai_jig_sequence_t;
+
+typedef struct {
+	lws_dll2_t		list;
+	lws_dll2_owner_t	gpio_owner;
+	lws_dll2_owner_t	seq_owner;
+
+	lws_sorted_usec_list_t	sul;		/* next step in ongoing seq */
+	sai_jig_seq_item_t	*current;	/* next seq step */
+
+	const char		*name;
+
+	struct lws		*wsi;
+} sai_jig_target_t;
+
+typedef struct {
+	lws_dll2_owner_t	target_owner;
+	struct gpiod_chip	*chip[16];
+	struct lwsac		*ac_conf;
+	int			port;
+	const char		*iface;
+	struct lws_context	*ctx;
+} sai_jig_t;
+
+/*
+ * We read the JSON config using lws_struct... instrument the related structures
+ */
+
+static const lws_struct_map_t lsm_sai_jig_gpio[] = {
+	LSM_UNSIGNED	(sai_jig_gpio_t, chip_idx,		"chip_idx"),
+	LSM_UNSIGNED	(sai_jig_gpio_t, offset,		"offset"),
+	LSM_UNSIGNED	(sai_jig_gpio_t, safe,			"safe"),
+	LSM_STRING_PTR	(sai_jig_gpio_t, name,			"name"),
+	LSM_STRING_PTR	(sai_jig_gpio_t, wire,			"wire"),
+};
+
+static const lws_struct_map_t lsm_sai_jig_seq_item[] = {
+	LSM_STRING_PTR	(sai_jig_seq_item_t, gpio_name,		"gpio_name"),
+	LSM_UNSIGNED	(sai_jig_seq_item_t, value,		"value"),
+};
+
+static const lws_struct_map_t lsm_sai_jig_sequence[] = {
+	LSM_STRING_PTR	(sai_jig_sequence_t, name,		"name"),
+	LSM_LIST	(sai_jig_sequence_t, seq_owner,
+			 sai_jig_seq_item_t, list,
+			 NULL, lsm_sai_jig_seq_item,		"seq"),
+};
+
+static const lws_struct_map_t lsm_sai_jig_target[] = {
+	LSM_STRING_PTR	(sai_jig_target_t, name,		"name"),
+	LSM_LIST	(sai_jig_target_t, gpio_owner, sai_jig_gpio_t, list,
+			 NULL, lsm_sai_jig_gpio,		"gpios"),
+	LSM_LIST	(sai_jig_target_t, seq_owner, sai_jig_sequence_t, list,
+			 NULL, lsm_sai_jig_sequence,		"sequences"),
+};
+
+static const lws_struct_map_t lsm_sai_jig[] = {
+	LSM_STRING_PTR	(sai_jig_t, iface,			"iface"),
+	LSM_UNSIGNED	(sai_jig_t, port,			"port"),
+	LSM_LIST	(sai_jig_t, target_owner, sai_jig_target_t, list,
+			 NULL, lsm_sai_jig_target,		"targets"),
+};
+
+static const lws_struct_map_t lsm_jig_schema[] = {
+        LSM_SCHEMA      (sai_jig_t, NULL, lsm_sai_jig,		"sai-jig"),
+};
+
+static const char * const jig_conf =
+"{"
+	"\"schema\":	\"sai-jig\","
+	"\"port\":		44000,"
+	"\"targets\":	["
+		"{"
+			"\"name\": \"linkit-7697-1\","
+                	"\"gpios\": ["
+                	        "{"
+					"\"chip_index\":	0,"
+					"\"name\":		\"nReset\","
+                                	"\"offset\":	17,"
+                                	"\"wire\":		\"RST\","
+					"\"safe\":		0"
+	                        "}, {"
+					"\"name\":		\"usr\","
+	                                "\"chip_index\":	0,"
+                                	"\"offset\":	22,"
+                                	"\"wire\":		\"P6\","
+					"\"safe\":		0"
+				"}"
+                        "], \"sequences\": ["
+                        	"{"
+					"\"name\":		\"reset\","
+					"\"seq\": ["
+		                                "{ \"gpio_name\": \"nReset\", 	\"value\": 0 },"
+		                                "{ \"gpio_name\": \"usr\",		\"value\": 0 },"
+	        	                        "{				\"value\": 300 },"
+		                                "{ \"gpio_name\": \"nReset\",	\"value\": 1 }"
+					"]"
+                        	"}, {"
+					"\"name\":		\"flash\","
+					"\"seq\": ["
+		                                "{ \"gpio_name\": \"nReset\",	\"value\": 0 },"
+		                                "{ \"gpio_name\": \"usr\",		\"value\": 1 },"
+	        	                        "{				\"value\": 300 },"
+		                                "{ \"gpio_name\": \"nReset\",	\"value\": 1 },"
+	        	                        "{				\"value\": 100 },"
+		                                "{ \"gpio_name\": \"usr\",		\"value\": 0 }"
+					"]"
+                        	"}"
+                	"]"
+		"}"
+	"]"
+"}";
+
+
+
+extern int test2(void);
+
 /*
  * in this example, the JSON is for one "builder" object, which may specify
  * a child list "targets" of zero or more "target" objects.
@@ -296,6 +437,63 @@ static const lws_struct_map_t lsm_schema_map[] = {
 			 lsm_other,		"com-warmcat-sai-other"),
 };
 
+typedef struct sai_cancel {
+	char task_uuid[65];
+} sai_cancel_t;
+
+const lws_struct_map_t lsm_task_cancel[] = {
+	LSM_CARRAY	(sai_cancel_t, task_uuid,	 "uuid"),
+};
+
+static const lws_struct_map_t t2_map[] = {
+	LSM_SCHEMA	(sai_cancel_t, NULL, lsm_task_cancel,
+					      "com.warmcat.sai.taskinfo"),
+	LSM_SCHEMA	(sai_cancel_t, NULL, lsm_task_cancel,
+					      "com.warmcat.sai.eventinfo"),
+	LSM_SCHEMA	(sai_cancel_t, NULL, lsm_task_cancel,
+			/* shares struct */   "com.warmcat.sai.taskreset"),
+	LSM_SCHEMA	(sai_cancel_t, NULL, lsm_task_cancel,
+			/* shares struct */   "com.warmcat.sai.eventreset"),
+	LSM_SCHEMA	(sai_cancel_t, NULL, lsm_task_cancel,
+			/* shares struct */   "com.warmcat.sai.eventdelete"),
+	LSM_SCHEMA	(sai_cancel_t,		 NULL, lsm_task_cancel,
+					      "com.warmcat.sai.taskcan"),
+};
+
+static const char *t2 =
+	"{\"schema\":\"com.warmcat.sai.taskcan\","
+	 "\"uuid\": \"071ab46ab4296e5de674c628fec17c55088254679f7714ad991f8c4873dca\"}\x01\x02\xff\xff\xff\xff";
+
+typedef struct xlws_wifi_creds {
+	lws_dll2_t	list;
+	char 		ssid[33];
+	char		passphrase[64];
+	int		alg;
+	char		bssid[6];
+} xlws_wifi_creds_t;
+
+typedef struct xlws_netdevs {
+	lws_dll2_owner_t	owner_creds;
+} xlws_netdevs_t;
+
+static const lws_struct_map_t lsm_wifi_creds[] = {
+	LSM_CARRAY	(xlws_wifi_creds_t, ssid,		"ssid"),
+	LSM_CARRAY	(xlws_wifi_creds_t, passphrase,		"passphrase"),
+	LSM_UNSIGNED	(xlws_wifi_creds_t, alg,			"alg"),
+	LSM_STRING_PTR	(xlws_wifi_creds_t, bssid,		"bssid"),
+};
+
+static const lws_struct_map_t lsm_netdev_credentials[] = {
+	LSM_LIST	(xlws_netdevs_t, owner_creds, xlws_wifi_creds_t, list,
+			 NULL, lsm_wifi_creds,			"credentials"),
+};
+
+static const lws_struct_map_t lsm_netdev_schema[] = {
+	LSM_SCHEMA	(xlws_netdevs_t, NULL, lsm_netdev_credentials,
+					      "com.warmcat.sai.taskinfo"),
+};
+
+
 static int
 show_target(struct lws_dll2 *d, void *user)
 {
@@ -345,8 +543,8 @@ int main(int argc, const char **argv)
 		a.ac_block_size = 512;
 
 		lws_struct_json_init_parse(&ctx, NULL, &a);
-		n = (int)(signed char)lejp_parse(&ctx, (uint8_t *)json_tests[m],
-						 strlen(json_tests[m]));
+		n = lejp_parse(&ctx, (uint8_t *)json_tests[m],
+						 (int)strlen(json_tests[m]));
 		if (n < 0) {
 			lwsl_err("%s: notification JSON decode failed '%s'\n",
 					__func__, lejp_error_to_string(n));
@@ -482,12 +680,122 @@ done:
 
 	lws_struct_json_serialize_destroy(&ser);
 
+	lwsl_notice("Test set 2\n");
+
+	memset(&a, 0, sizeof(a));
+	a.map_st[0] = t2_map;
+	a.map_entries_st[0] = LWS_ARRAY_SIZE(t2_map);
+	a.ac_block_size = 128;
+
+	lws_struct_json_init_parse(&ctx, NULL, &a);
+	m = lejp_parse(&ctx, (uint8_t *)t2, (int)strlen(t2));
+	if (m < 0 || !a.dest) {
+		lwsl_notice("%s: notification JSON decode failed '%s'\n",
+				__func__, lejp_error_to_string(m));
+		goto bail;
+	}
+
+	lwsl_notice("Test set 2: %d: %s\n", m,
+			((sai_cancel_t *)a.dest)->task_uuid);
+
+	lwsac_free(&a.ac);
+
+	if (test2())
+		goto bail;
+
+	{
+		lws_struct_serialize_t *js;
+		xlws_wifi_creds_t creds;
+		xlws_netdevs_t netdevs;
+		unsigned char *buf;
+		size_t w;
+		int n;
+
+		memset(&creds, 0, sizeof(creds));
+		memset(&netdevs, 0, sizeof(netdevs));
+
+		lws_strncpy(creds.ssid, "xxx", sizeof(creds.ssid));
+		lws_strncpy(creds.passphrase, "yyy", sizeof(creds.passphrase));
+		lws_dll2_add_tail(&creds.list, &netdevs.owner_creds);
+
+		buf = malloc(2048); /* length should be computed */
+
+		js = lws_struct_json_serialize_create(lsm_netdev_schema,
+			LWS_ARRAY_SIZE(lsm_netdev_schema), 0, &netdevs);
+		if (!js)
+			goto bail;
+
+		n = lws_struct_json_serialize(js, buf, 2048, &w);
+		lws_struct_json_serialize_destroy(&js);
+		if (n != LSJS_RESULT_FINISH)
+			goto bail;
+		if (strcmp("{\"schema\":\"com.warmcat.sai.taskinfo\",\"credentials\":[{\"ssid\":\"xxx\",\"passphrase\":\"yyy\",\"alg\":0}]}", (const char *)buf)) {
+			puts((const char *)buf);
+			goto bail;
+		}
+		free(buf);
+	}
+
+	{
+		struct x { lws_dll2_t list; const char *sz; };
+		struct x x1, x2, *xp;
+		lws_dll2_owner_t o;
+
+		lws_dll2_owner_clear(&o);
+		memset(&x1, 0, sizeof(x1));
+		memset(&x2, 0, sizeof(x2));
+
+		x1.sz = "nope";
+		x2.sz = "yes";
+
+		lws_dll2_add_tail(&x1.list, &o);
+		lws_dll2_add_tail(&x2.list, &o);
+
+		xp = lws_dll2_search_sz_pl(&o, "yes", 3, struct x, list, sz);
+		if (xp != &x2) {
+			lwsl_err("%s: 1 xp %p\n", __func__, xp);
+			goto bail;
+		}
+		xp = lws_dll2_search_sz_pl(&o, "nope", 4, struct x, list, sz);
+		if (xp != &x1) {
+			lwsl_err("%s: 2 xp %p\n", __func__, xp);
+			goto bail;
+		}
+		xp = lws_dll2_search_sz_pl(&o, "wrong", 4, struct x, list, sz);
+		if (xp) {
+			lwsl_err("%s: 3 xp %p\n", __func__, xp);
+			goto bail;
+		}
+	}
+
+	{
+		lws_struct_args_t a;
+		struct lejp_ctx ctx;
+		int m;
+
+		memset(&a, 0, sizeof(a));
+		a.map_st[0] = lsm_jig_schema;
+		a.map_entries_st[0] = LWS_ARRAY_SIZE(lsm_jig_schema);
+		a.ac_block_size = 512;
+
+		lws_struct_json_init_parse(&ctx, NULL, &a);
+
+		m = lejp_parse(&ctx, (uint8_t *)jig_conf, (int)strlen(jig_conf));
+
+		if (m < 0 || !a.dest) {
+			lwsl_err("%s: line %d: JSON decode failed '%s'\n",
+				    __func__, ctx.line, lejp_error_to_string(m));
+			goto bail;
+		}
+	}
 
 	lwsl_user("Completed: PASS\n");
 
 	return 0;
 
 bail:
+if (test2())
+	return 1;
 	lwsl_user("Completed: FAIL\n");
 
 	return 1;
