@@ -17,7 +17,16 @@
 #include <signal.h>
 
 static int interrupted, bad = 1, status;
+#if defined(LWS_WITH_HTTP2)
+static int long_poll;
+#endif
 static struct lws *client_wsi;
+static const char *ba_user, *ba_password;
+
+static const lws_retry_bo_t retry = {
+	.secs_since_valid_ping = 3,
+	.secs_since_valid_hangup = 10,
+};
 
 static int
 callback_http(struct lws *wsi, enum lws_callback_reasons reason,
@@ -29,17 +38,60 @@ callback_http(struct lws *wsi, enum lws_callback_reasons reason,
 	case LWS_CALLBACK_CLIENT_CONNECTION_ERROR:
 		lwsl_err("CLIENT_CONNECTION_ERROR: %s\n",
 			 in ? (char *)in : "(null)");
-		client_wsi = NULL;
+		interrupted = 1;
 		break;
 
 	case LWS_CALLBACK_ESTABLISHED_CLIENT_HTTP:
-		status = lws_http_client_http_response(wsi);
-		lwsl_user("Connected with server response: %d\n", status);
+		{
+			char buf[128];
+
+			lws_get_peer_simple(wsi, buf, sizeof(buf));
+			status = lws_http_client_http_response(wsi);
+
+			lwsl_user("Connected to %s, http response: %d\n",
+					buf, status);
+		}
+#if defined(LWS_WITH_HTTP2)
+		if (long_poll) {
+			lwsl_user("%s: Client entering long poll mode\n", __func__);
+			lws_h2_client_stream_long_poll_rxonly(wsi);
+		}
+#endif
 		break;
 
+#if defined(LWS_WITH_HTTP_BASIC_AUTH)
+
+	/* you only need this if you need to do Basic Auth */
+	case LWS_CALLBACK_CLIENT_APPEND_HANDSHAKE_HEADER:
+	{
+		unsigned char **p = (unsigned char **)in, *end = (*p) + len;
+		char b[128];
+
+		if (!ba_user || !ba_password)
+			break;
+
+		if (lws_http_basic_auth_gen(ba_user, ba_password, b, sizeof(b)))
+			break;
+		if (lws_add_http_header_by_token(wsi, WSI_TOKEN_HTTP_AUTHORIZATION,
+				(unsigned char *)b, (int)strlen(b), p, end))
+			return -1;
+
+		break;
+	}
+#endif
+
 	/* chunks of chunked content, with header removed */
 	case LWS_CALLBACK_RECEIVE_CLIENT_HTTP_READ:
 		lwsl_user("RECEIVE_CLIENT_HTTP_READ: read %d\n", (int)len);
+#if defined(LWS_WITH_HTTP2)
+		if (long_poll) {
+			char dotstar[128];
+			lws_strnncpy(dotstar, (const char *)in, len,
+				     sizeof(dotstar));
+			lwsl_notice("long poll rx: %d '%s'\n", (int)len,
+					dotstar);
+		}
+#endif
 #if 0  /* enable to dump the html */
 		{
 			const char *p = in;
@@ -67,13 +119,13 @@ callback_http(struct lws *wsi, enum lws_callback_reasons reason,
 
 	case LWS_CALLBACK_COMPLETED_CLIENT_HTTP:
 		lwsl_user("LWS_CALLBACK_COMPLETED_CLIENT_HTTP\n");
-		client_wsi = NULL;
+		interrupted = 1;
 		bad = status != 200;
 		lws_cancel_service(lws_get_context(wsi)); /* abort poll wait */
 		break;
 
 	case LWS_CALLBACK_CLOSED_CLIENT_HTTP:
-		client_wsi = NULL;
+		interrupted = 1;
 		bad = status != 200;
 		lws_cancel_service(lws_get_context(wsi)); /* abort poll wait */
 		break;
@@ -101,34 +153,136 @@ sigint_handler(int sig)
 	interrupted = 1;
 }
 
+struct args {
+	int argc;
+	const char **argv;
+};
+
+static int
+system_notify_cb(lws_state_manager_t *mgr, lws_state_notify_link_t *link,
+		   int current, int target)
+{
+	struct lws_context *context = mgr->parent;
+	struct lws_client_connect_info i;
+	struct args *a = lws_context_user(context);
+	const char *p;
+
+	if (current != LWS_SYSTATE_OPERATIONAL || target != LWS_SYSTATE_OPERATIONAL)
+		return 0;
+
+	lwsl_info("%s: operational\n", __func__);
+
+	memset(&i, 0, sizeof i); /* otherwise uninitialized garbage */
+	i.context = context;
+	if (!lws_cmdline_option(a->argc, a->argv, "-n")) {
+		i.ssl_connection = LCCSCF_USE_SSL;
+#if defined(LWS_WITH_HTTP2)
+		/* requires h2 */
+		if (lws_cmdline_option(a->argc, a->argv, "--long-poll")) {
+			lwsl_user("%s: long poll mode\n", __func__);
+			long_poll = 1;
+		}
+#endif
+	}
+
+	if (lws_cmdline_option(a->argc, a->argv, "-l")) {
+		i.port = 7681;
+		i.address = "localhost";
+		i.ssl_connection |= LCCSCF_ALLOW_SELFSIGNED;
+	} else {
+		i.port = 443;
+		i.address = "warmcat.com";
+	}
+
+	if (lws_cmdline_option(a->argc, a->argv, "--nossl"))
+		i.ssl_connection = 0;
+
+	i.ssl_connection |= LCCSCF_H2_QUIRK_OVERFLOWS_TXCR |
+			    LCCSCF_H2_QUIRK_NGHTTP2_END_STREAM;
+
+	i.alpn = "h2";
+	if (lws_cmdline_option(a->argc, a->argv, "--h1"))
+		i.alpn = "http/1.1";
+
+	if (lws_cmdline_option(a->argc, a->argv, "--h2-prior-knowledge"))
+		i.ssl_connection |= LCCSCF_H2_PRIOR_KNOWLEDGE;
+
+	if ((p = lws_cmdline_option(a->argc, a->argv, "-p")))
+		i.port = atoi(p);
+
+	if ((p = lws_cmdline_option(a->argc, a->argv, "--user")))
+		ba_user = p;
+	if ((p = lws_cmdline_option(a->argc, a->argv, "--password")))
+		ba_password = p;
+
+	if (lws_cmdline_option(a->argc, a->argv, "-j"))
+		i.ssl_connection |= LCCSCF_ALLOW_SELFSIGNED;
+
+	if (lws_cmdline_option(a->argc, a->argv, "-k"))
+		i.ssl_connection |= LCCSCF_ALLOW_INSECURE;
+
+	if (lws_cmdline_option(a->argc, a->argv, "-m"))
+		i.ssl_connection |= LCCSCF_SKIP_SERVER_CERT_HOSTNAME_CHECK;
+
+	if (lws_cmdline_option(a->argc, a->argv, "-e"))
+		i.ssl_connection |= LCCSCF_ALLOW_EXPIRED;
+
+	if ((p = lws_cmdline_option(a->argc, a->argv, "-f"))) {
+		i.ssl_connection |= LCCSCF_H2_MANUAL_RXFLOW;
+		i.manual_initial_tx_credit = atoi(p);
+		lwsl_notice("%s: manual peer tx credit %d\n", __func__,
+				i.manual_initial_tx_credit);
+	}
+
+	/* the default validity check is 5m / 5m10s... -v = 3s / 10s */
+
+	if (lws_cmdline_option(a->argc, a->argv, "-v"))
+		i.retry_and_idle_policy = &retry;
+
+	if ((p = lws_cmdline_option(a->argc, a->argv, "--server")))
+		i.address = p;
+
+	if ((p = lws_cmdline_option(a->argc, a->argv, "--path")))
+		i.path = p;
+	else
+		i.path = "/";
+
+	i.host = i.address;
+	i.origin = i.address;
+	i.method = "GET";
+
+	i.protocol = protocols[0].name;
+	i.pwsi = &client_wsi;
+
+	return !lws_client_connect_via_info(&i);
+}
+
 int main(int argc, const char **argv)
 {
+	lws_state_notify_link_t notifier = { {0}, system_notify_cb, "app" };
+	lws_state_notify_link_t *na[] = { &notifier, NULL };
 	struct lws_context_creation_info info;
-	struct lws_client_connect_info i;
 	struct lws_context *context;
-	const char *p;
-	int n = 0, logs = LLL_USER | LLL_ERR | LLL_WARN | LLL_NOTICE
-		   /*
-		    * For LLL_ verbosity above NOTICE to be built into lws,
-		    * lws must have been configured and built with
-		    * -DCMAKE_BUILD_TYPE=DEBUG instead of =RELEASE
-		    *
-		    * | LLL_INFO   | LLL_PARSER  | LLL_HEADER | LLL_EXT |
-		    *   LLL_CLIENT | LLL_LATENCY | LLL_DEBUG
-		    */ ;
+	struct args args;
+	int n = 0;
+	// uint8_t memcert[4096];
+
+	args.argc = argc;
+	args.argv = argv;
 
 	signal(SIGINT, sigint_handler);
 
-	if ((p = lws_cmdline_option(argc, argv, "-d")))
-		logs = atoi(p);
+	memset(&info, 0, sizeof info); /* otherwise uninitialized garbage */
+	lws_cmdline_option_handle_builtin(argc, argv, &info);
 
-	lws_set_log_level(logs, NULL);
 	lwsl_user("LWS minimal http client [-d<verbosity>] [-l] [--h1]\n");
 
-	memset(&info, 0, sizeof info); /* otherwise uninitialized garbage */
 	info.options = LWS_SERVER_OPTION_DO_SSL_GLOBAL_INIT;
 	info.port = CONTEXT_PORT_NO_LISTEN; /* we do not run any server */
 	info.protocols = protocols;
+	info.user = &args;
+	info.register_notifier_list = na;
+       info.connect_timeout_secs = 30;
 
 	/*
 	 * since we know this lws context is only ever going to be used with
@@ -139,65 +293,30 @@ int main(int argc, const char **argv)
 	 */
 	info.fd_limit_per_thread = 1 + 1 + 1;
 
-#if defined(LWS_WITH_MBEDTLS)
+#if defined(LWS_WITH_MBEDTLS) || defined(USE_WOLFSSL)
 	/*
 	 * OpenSSL uses the system trust store.  mbedTLS has to be told which
 	 * CA to trust explicitly.
 	 */
 	info.client_ssl_ca_filepath = "./warmcat.com.cer";
 #endif
-
+#if 0
+	n = open("./warmcat.com.cer", O_RDONLY);
+	if (n >= 0) {
+		info.client_ssl_ca_mem_len = read(n, memcert, sizeof(memcert));
+		info.client_ssl_ca_mem = memcert;
+		close(n);
+		n = 0;
+		memcert[info.client_ssl_ca_mem_len++] = '\0';
+	}
+#endif
 	context = lws_create_context(&info);
 	if (!context) {
 		lwsl_err("lws init failed\n");
 		return 1;
 	}
 
-	memset(&i, 0, sizeof i); /* otherwise uninitialized garbage */
-	i.context = context;
-	if (!lws_cmdline_option(argc, argv, "-n"))
-		i.ssl_connection = LCCSCF_USE_SSL;
-
-	if (lws_cmdline_option(argc, argv, "-l")) {
-		i.port = 7681;
-		i.address = "localhost";
-		i.ssl_connection |= LCCSCF_ALLOW_SELFSIGNED;
-	} else {
-		i.port = 443;
-		i.address = "warmcat.com";
-	}
-
-	if (lws_cmdline_option(argc, argv, "--h1"))
-		i.alpn = "http/1.1";
-
-	if ((p = lws_cmdline_option(argc, argv, "-p")))
-		i.port = atoi(p);
-
-	if (lws_cmdline_option(argc, argv, "-j"))
-		i.ssl_connection |= LCCSCF_ALLOW_SELFSIGNED;
-
-	if (lws_cmdline_option(argc, argv, "-k"))
-		i.ssl_connection |= LCCSCF_ALLOW_INSECURE;
-
-	if (lws_cmdline_option(argc, argv, "-m"))
-		i.ssl_connection |= LCCSCF_SKIP_SERVER_CERT_HOSTNAME_CHECK;
-
-	if (lws_cmdline_option(argc, argv, "-e"))
-		i.ssl_connection |= LCCSCF_ALLOW_EXPIRED;
-
-	if ((p = lws_cmdline_option(argc, argv, "--server")))
-		i.address = p;
-
-	i.path = "/";
-	i.host = i.address;
-	i.origin = i.address;
-	i.method = "GET";
-
-	i.protocol = protocols[0].name;
-	i.pwsi = &client_wsi;
-	lws_client_connect_via_info(&i);
-
-	while (n >= 0 && client_wsi && !interrupted)
+	while (n >= 0 && !interrupted)
 		n = lws_service(context, 0);
 
 	lws_context_destroy(context);
