@@ -245,7 +245,7 @@ static const struct lws_protocols protocols[] = {
 	LWS_PLUGIN_PROTOCOL_LWS_STATUS
 };
 
-LWS_VISIBLE const lws_plugin_protocol_t protocol_lws_status = {
+LWS_VISIBLE const lws_plugin_protocol_t lws_status = {
 	.hdr = {
 		"lws status",
 		"lws_protocol_plugin",
